@@ -1,6 +1,6 @@
 package SADI::Simple::ServiceBase;
 {
-  $SADI::Simple::ServiceBase::VERSION = '0.011';
+  $SADI::Simple::ServiceBase::VERSION = '0.012';
 }
 
 use strict;
@@ -157,7 +157,7 @@ sub _type_outputs
         my $uri = $s->uri();
         if ($input_uri_hash{$uri} && !$visited_output_uris{$uri}) {
             my $statement = RDF::Trine::Statement->new($s, $rdf_type, $output_type);
-            $output_model->add_statement($statement);
+            $output_model->add_statement($statement, $s);  # need to add context here, for nanopublications
             $visited_output_uris{$uri} = 1;
         }
     }
